"use strict";
const mongoose = require('mongoose');
class CCService {
    constructor(modelName) {
        this.modelName = modelName;
        if (!this.modelName)
            throw "modelName field must be set";
        this.model = mongoose.model(this.modelName);
    }
    /**
     * Retrieves all documents from the collection
     */
    getAll(callback) {
        return this.model.find().exec(callback);
    }
    /**
     * Retrieves a document by it's Id
     */
    byId(id, callback) {
        return this.model.findById(id).exec(callback);
    }
    /**
     * Updates the document by the given id and returns it
     * @param {string | number | ObjectId} id the Id of the document in the db
     * @param  {any} update a hashmap of the fields to update and their new values as keys
     * @param {Clb<T>}  callback optional callback that can be used to retrieve the results
     * @returns {Primise<T>}
     */
    updateById(id, update, callback) {
        return this.model.findByIdAndUpdate(id, update, { new: true }).exec(callback);
    }
    /**
     * Deletes a document by the given id and returns it
     */
    deleteById(id, callback) {
        return this.model.findByIdAndRemove(id).exec(callback);
    }
    /**
     * For a given object it tries to create T instance, saves it to the database and returns it
     * @argument document:any
     * @returns T
     */
    createAndSave(document, callback) {
        return this.insert(this.create(document), callback);
    }
    /**
     * From a given array of objects it tries to create T[] array and saves it to the database
     */
    createAndSaveMany(documents, callback) {
        return this.model.create(documents, callback);
    }
    /**
     * Creates a new object of type T and returns it.
     * @param {any} document a hasmap with the fields we want to create the object from
     * @param  {Clb<T>} an optional callback function that can be used to get the newly created object
     */
    create(document = {}, callback) {
        let newDoc = new this.model(document);
        if (callback)
            callback(null, newDoc);
        return newDoc;
    }
    insert(document, callback) {
        return document.save(callback);
    }
}
exports.CCService = CCService;
